@@ -3,20 +3,27 @@
 # Copyright (c) 2015-2022 ODC Contributors
 # SPDX-License-Identifier: Apache-2.0
 import datetime
+import logging
 from pathlib import Path
+from threading import Lock
 
 from abc import ABC, abstractmethod
 from typing import (Any, Iterable, Iterator,
                     List, Mapping, Optional,
-                    Tuple, Union)
+                    Tuple, Union, Sequence)
 from uuid import UUID
 
 from datacube.config import LocalConfig
+from datacube.index.exceptions import TransactionException
 from datacube.index.fields import Field
 from datacube.model import Dataset, MetadataType, Range
 from datacube.model import DatasetType as Product
 from datacube.utils import cached_property, read_documents, InvalidDocException
 from datacube.utils.changes import AllowPolicy, Change, Offset
+from datacube.utils.generic import thread_local_cache
+from datacube.utils.geometry import CRS, Geometry, box
+
+_LOG = logging.getLogger(__name__)
 
 
 class AbstractUserResource(ABC):
@@ -238,9 +245,11 @@ class AbstractMetadataTypeResource(ABC):
         :returns: All available MetadataType models
         """
 
+
 QueryField = Union[str, float, int, Range, datetime.datetime]
 QueryDict = Mapping[str, QueryField]
 
+
 class AbstractProductResource(ABC):
     """
     Abstract base class for the Product portion of an index api.
@@ -539,9 +548,8 @@ class AbstractDatasetResource(ABC):
         """
 
     @abstractmethod
-    def add(self,
-            dataset: Dataset,
-            with_lineage: Optional[bool] = None
+    def add(self, dataset: Dataset,
+            with_lineage: bool = True
            ) -> Dataset:
         """
         Add ``dataset`` to the index. No-op if it is already present.
@@ -549,7 +557,7 @@ class AbstractDatasetResource(ABC):
         :param dataset: Unpersisted dataset model
 
         :param with_lineage:
-           - ``True|None`` attempt adding lineage datasets if missing
+           - ``True (default)`` attempt adding lineage datasets if missing
            - ``False`` record lineage relations, but do not attempt
              adding lineage datasets to the db
 
@@ -756,7 +764,7 @@ class AbstractDatasetResource(ABC):
         """
         Perform a search, returning results as Dataset objects.
 
-        :param limit: Limit number of dataset (None/default = unlimited)
+        :param limit: Limit number of datasets per product (None/default = unlimited)
         :param query: search query parameters
         :return: Matching datasets
         """
@@ -901,6 +909,244 @@ class AbstractDatasetResource(ABC):
         property functions).
         """
 
+    @abstractmethod
+    def spatial_extent(self, ids: Iterable[DSID], crs: CRS = CRS("EPSG:4326")) -> Optional[Geometry]:
+        """
+        Return the combined spatial extent of the nominated datasets.
+
+        Uses spatial index.
+        Returns None if no index for the CRS, or if no identified datasets are indexed in the relevant spatial index.
+        Result will not include extents of datasets that cannot be validly projected into the CRS.
+
+        :param ids: An iterable of dataset IDs
+        :param crs: A CRS (defaults to EPSG:4326)
+        :return: The combined spatial extents of the datasets.
+        """
+
+    def _extract_geom_from_query(self, q: Mapping[str, QueryField]) -> Optional[Geometry]:
+        """
+        Utility method for index drivers supporting spatial indexes.
+
+        Extract a Geometry from a dataset query.  Backwards compatible with old lat/lon style queries.
+
+        :param q: A query dictionary
+        :return: A polygon or multipolygon type Geometry.  None if no spatial query clauses.
+        """
+        geom: Optional[Geometry] = None
+        if "geometry" in q:
+            # New geometry-style spatial query
+            geom_term = q.pop("geometry")
+            try:
+                geom = Geometry(geom_term)
+            except ValueError:
+                # Can't convert to single Geometry. If it is an iterable of Geometries, return the union
+                for term in geom_term:
+                    if geom is None:
+                        geom = Geometry(term)
+                    else:
+                        geom = geom.union(Geometry(term))
+            if "lat" in q or "lon" in q:
+                raise ValueError("Cannot specify lat/lon AND geometry in the same query")
+            assert geom.crs
+        else:
+            # Old lat/lon--style spatial query (or no spatial query)
+            lat = q.pop("lat", None)
+            lon = q.pop("lon", None)
+            if lat is None and lon is None:
+                # No spatial query
+                _LOG.info("No spatial query")
+                return None
+
+            # Old lat/lon--style spatial query
+            if lat is None:
+                lat = Range(begin=-90, end=90)
+            if lon is None:
+                lon = Range(begin=-180, end=180)
+            delta = 0.000001
+            if isinstance(lat, Range) and isinstance(lon, Range):
+                # ranges for both - build a box.
+                geom = box(lon.begin, lat.begin, lon.end, lat.end, crs=CRS("EPSG:4326"))
+            elif isinstance(lat, Range):
+                if isinstance(lon, (int, float)):
+                    # lat is a range, but lon is scalar - geom is ideally a line
+                    # datacube.utils.geometry is always (x, y) order - ignore lat,lon order specified by EPSG:4326
+                    geom = box(lon - delta, lat.begin, lon + delta, lat.end, crs=CRS("EPSG:4326"))
+                else:
+                    raise ValueError("lon search term must be a Range or a numeric scalar")
+            elif isinstance(lon, Range):
+                if isinstance(lat, (int, float)):
+                    # lon is a range, but lat is scalar - geom is ideally a line
+                    # datacube.utils.geometry is always (x, y) order - ignore lat,lon order specified by EPSG:4326
+                    geom = box(lon.begin, lat - delta, lon.end, lat + delta, crs=CRS("EPSG:4326"))
+                else:
+                    raise ValueError("lat search term must be a Range or a numeric scalar")
+            else:
+                if isinstance(lon, (int, float)) and isinstance(lat, (int, float)):
+                    # Lat and Lon are both scalars - geom is ideally point
+                    # datacube.utils.geometry is always (x, y) order - ignore lat,lon order specified by EPSG:4326
+                    geom = box(lon - delta, lat - delta, lon + delta, lat + delta, crs=CRS("EPSG:4326"))
+                else:
+                    raise ValueError("lat and lon search terms must be of type Range or a numeric scalar")
+        _LOG.info("Spatial Query Geometry: %s", geom.wkt)
+        return geom
+
+
+class AbstractTransaction(ABC):
+    """
+    Abstract base class for a Transaction Manager.  All index implementations should extend this base class.
+
+    Thread-local storage and locks ensures one active transaction per index per thread.
+    """
+
+    def __init__(self, index_id: str):
+        self._connection: Any = None
+        self._tls_id = f"txn-{index_id}"
+        self._obj_lock = Lock()
+
+    # Main Transaction API
+    def begin(self) -> None:
+        """
+        Start a new transaction.
+
+        Raises an error if a transaction is already active for this thread.
+
+        Calls implementation-specific _new_connection() method and manages thread local storage and locks.
+        """
+        with self._obj_lock:
+            if self._connection is not None:
+                raise ValueError("Cannot start a new transaction as one is already active")
+            self._tls_stash()
+
+    def commit(self) -> None:
+        """
+        Commit the transaction.
+
+        Raises an error if transaction is not active.
+
+        Calls implementation-specific _commit() method, and manages thread local storage and locks.
+        """
+        with self._obj_lock:
+            if self._connection is None:
+                raise ValueError("Cannot commit inactive transaction")
+            self._commit()
+            self._release_connection()
+            self._connection = None
+            self._tls_purge()
+
+    def rollback(self) -> None:
+        """
+        Rollback the transaction.
+
+        Raises an error if transaction is not active.
+
+        Calls implementation-specific _rollback() method, and manages thread local storage and locks.
+        """
+        with self._obj_lock:
+            if self._connection is None:
+                raise ValueError("Cannot rollback inactive transaction")
+            self._rollback()
+            self._release_connection()
+            self._connection = None
+            self._tls_purge()
+
+    @property
+    def active(self):
+        """
+        :return:  True if the transaction is active.
+        """
+        return self._connection is not None
+
+    # Manage thread-local storage
+    def _tls_stash(self) -> None:
+        """
+        Check TLS is empty, create a new connection and stash it.
+        :return:
+        """
+        stored_val = thread_local_cache(self._tls_id)
+        if stored_val is not None:
+            raise ValueError("Cannot start a new transaction as one is already active for this thread")
+        self._connection = self._new_connection()
+        thread_local_cache(self._tls_id, purge=True)
+        thread_local_cache(self._tls_id, self)
+
+    def _tls_purge(self) -> None:
+        thread_local_cache(self._tls_id, purge=True)
+
+    # Commit/Rollback exceptions for Context Manager usage patterns
+    def commit_exception(self, errmsg: str) -> TransactionException:
+        return TransactionException(errmsg, commit=True)
+
+    def rollback_exception(self, errmsg: str) -> TransactionException:
+        return TransactionException(errmsg, commit=False)
+
+    # Context Manager Interface
+    def __enter__(self):
+        self.begin()
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if not self.active:
+            # User has already manually committed or rolled back.
+            return True
+        if exc_type is not None and issubclass(exc_type, TransactionException):
+            # User raised a TransactionException,  Commit or rollback as per exception
+            if exc_value.commit:
+                self.commit()
+            else:
+                self.rollback()
+            # Tell runtime exception is caught and handled.
+            return True
+        elif exc_value is not None:
+            # Any other exception - rollback
+            self.rollback()
+            # Instruct runtime to rethrow exception
+            return False
+        else:
+            # Exited without exception - commit and continue
+            self.commit()
+            return True
+
+    # Internal abstract methods for implementation-specific functionality
+    @abstractmethod
+    def _new_connection(self) -> Any:
+        """
+        :return: a new index driver object representing a database connection or equivalent against which transactions
+        will be executed.
+        """
+
+    @abstractmethod
+    def _commit(self) -> None:
+        """
+        Commit the transaction.
+        """
+
+    @abstractmethod
+    def _rollback(self) -> None:
+        """
+        Rollback the transaction.
+        """
+
+    @abstractmethod
+    def _release_connection(self) -> None:
+        """
+        Release the connection object stored in self._connection
+        """
+
+
+class UnhandledTransaction(AbstractTransaction):
+    # Minimal implementation for index drivers with no transaction handling.
+    def _new_connection(self) -> Any:
+        return True
+
+    def _commit(self) -> None:
+        pass
+
+    def _rollback(self) -> None:
+        pass
+
+    def _release_connection(self) -> None:
+        pass
+
 
 class AbstractIndex(ABC):
     """
@@ -908,25 +1154,43 @@ class AbstractIndex(ABC):
     inherit from this base class and implement all abstract methods.
     """
 
-    @property
-    @abstractmethod
-    def url(self) -> str: pass
+    # Interface contracts
+    #   supports add() update() remove() etc methods.
+    supports_persistance = True
+    #   supports legacy ODCv1 EO style metadata types.
+    supports_legacy = True
+    #   supports non-geospatial (e.g. telemetry) metadata types
+    supports_nongeo = True
+    #   supports lineage
+    supports_lineage = True
+    supports_source_filters = True
+    # Supports ACID transactions
+    supports_transactions = False
 
     @property
     @abstractmethod
-    def users(self) -> AbstractUserResource: pass
+    def url(self) -> str:
+        """A string representing the index"""
 
     @property
     @abstractmethod
-    def metadata_types(self) -> AbstractMetadataTypeResource: pass
+    def users(self) -> AbstractUserResource:
+        """A User Resource instance for the index"""
 
     @property
     @abstractmethod
-    def products(self) -> AbstractProductResource: pass
+    def metadata_types(self) -> AbstractMetadataTypeResource:
+        """A MetadataType Resource instance for the index"""
 
     @property
     @abstractmethod
-    def datasets(self) -> AbstractDatasetResource: pass
+    def products(self) -> AbstractProductResource:
+        """A Product Resource instance for the index"""
+
+    @property
+    @abstractmethod
+    def datasets(self) -> AbstractDatasetResource:
+        """A Dataset Resource instance for the index"""
 
     @classmethod
     @abstractmethod
@@ -935,22 +1199,95 @@ class AbstractIndex(ABC):
                     application_name: Optional[str] = None,
                     validate_connection: bool = True
                    ) -> "AbstractIndex":
-        pass
+        """Instantiate a new index from a LocalConfig object"""
 
     @classmethod
     @abstractmethod
     def get_dataset_fields(cls,
                            doc: dict
                           ) -> Mapping[str, Field]:
-        pass
+        """Return dataset search fields from a metadata type document"""
 
     @abstractmethod
     def init_db(self,
                 with_default_types: bool = True,
-                with_permissions: bool = True) -> bool: pass
+                with_permissions: bool = True) -> bool:
+        """
+        Initialise an empty database.
+
+        :param with_default_types: Whether to create default metadata types
+        :param with_permissions: Whether to create db permissions
+        :return: true if the database was created, false if already exists
+        """
 
     @abstractmethod
-    def close(self) -> None: pass
+    def close(self) -> None:
+        """
+        Close and cleanup the Index.
+        """
+
+    @property
+    @abstractmethod
+    def index_id(self) -> str:
+        """
+        :return: Unique ID for this index
+                 (e.g. same database/dataset storage + same index driver implementation = same id)
+        """
+
+    @abstractmethod
+    def transaction(self) -> AbstractTransaction:
+        """
+        :return: a Transaction context manager for this index.
+        """
+
+    @abstractmethod
+    def create_spatial_index(self, crs: CRS) -> bool:
+        """
+        Create a spatial index using the nominated CRS.
+
+        :param crs: The CRS to use in the spatial index.
+        :return: True is the index was successfully created or already exists.
+                 None if spatial indexes are not supported.
+        """
+
+    def thread_transaction(self) -> Optional["AbstractTransaction"]:
+        """
+        :return: The existing Transaction object cached in thread-local storage for this index, if there is one.
+        """
+        return thread_local_cache(f"txn-{self.index_id}", None)
+
+    def spatial_indexes(self, refresh=False) -> Iterable[CRS]:
+        """
+        Return a list of CRSs for which spatiotemporal indexes exist in the database.
+
+        :param refresh: If true, re-read from database record (e.g. to catch spatial
+                        indexes recently created in another datacube session.
+        :return:
+        """
+        _LOG.warning("Spatial index API is unstable and may change between releases.")
+        return []
+
+    def update_spatial_index(self,
+                             crses: Sequence[CRS] = [],
+                             product_names: Sequence[str] = [],
+                             dataset_ids: Sequence[DSID] = []
+                             ) -> int:
+        """
+        Update a spatial index
+        :param crs: CRSs for Spatial Indexes to update. Default=all indexes
+        :param product_names: Product names to update
+        :param dsids: Dataset IDs to update
+
+        If neither product_names nor dataset ids are supplied, update for all datasets.
+
+        If both are supplied, both the named products and identified datasets are updated.
+
+        If spatial indexes are not supported by the index driver, always return zero.
+
+        :return:  Number of spatial index entries updated or verified as unindexed.
+        """
+        _LOG.warning("Spatial index API is unstable and may change between releases.")
+        return 0
 
     def __enter__(self):
         return self
@@ -970,14 +1307,13 @@ class AbstractIndexDriver(ABC):
                          application_name: Optional[str] = None,
                          validate_connection: bool = True
                         ) -> "AbstractIndex":
-        pass
+        ...
 
     @staticmethod
     @abstractmethod
-    def metadata_type_from_doc(
-                               definition: dict
+    def metadata_type_from_doc(definition: dict
                               ) -> MetadataType:
-        pass
+        ...
 
 
 # The special handling of grid_spatial, etc appears to NOT apply to EO3.
@@ -1003,4 +1339,4 @@ class DatasetSpatialMixin:
 
     @property
     def bounds(self):
-        return Dataset.bounds.__get__(self)
\ No newline at end of file
+        return Dataset.bounds.__get__(self)
