@@ -7,17 +7,16 @@
 # TODO: typehints need attention
 """ Tools for working with EO3 metadata
 """
-from types import SimpleNamespace
 from affine import Affine
-import toolz  # type: ignore[import]
-from typing import Dict, Any, Optional
+from functools import reduce
+from typing import Dict, Any, Iterable, Optional, Tuple, Union
+from uuid import UUID
 
 from datacube.utils.geometry import (
     SomeCRS,
     CRS,
     Geometry,
     polygon,
-    bbox_union,
     CoordList,
     BoundingBox,
     lonlat_bounds,
@@ -26,69 +25,68 @@ from datacube.utils.geometry import (
 EO3_SCHEMA = "https://schemas.opendatacube.org/dataset"
 
 
-def _norm_grid(grid: Dict[str, Any]) -> Any:
-    shape = grid.get('shape')
-    transform = grid.get('transform')
-    if shape is None or transform is None:
-        raise ValueError("Each grid must have .shape and .transform")
-    return SimpleNamespace(shape=shape,
-                           transform=Affine(*transform[:6]))
+class EO3Grid:
+    def __init__(self, grid: Dict[str, Any]) -> None:
+        shape = grid.get("shape")
+        if shape is None:
+            raise ValueError("Each grid must have a shape")
+        if len(shape) != 2:
+            raise ValueError("Grid shape must be two dimensional")
+        self.shape: Tuple[int, int] = tuple(int(x) for x in shape)
+        xform = grid.get("transform")
+        if xform is None:
+            raise ValueError("Each grid must have a transform")
+        if len(xform) != 6 and len(xform) != 9:
+            raise ValueError("Grid transform must have 6 or 9 elements.")
+        for elem in xform:
+            if type(elem) not in (int, float):
+                raise ValueError("All grid transform elements must be numbers")
+        if len(xform) == 9 and list(xform[6:]) != [0, 0, 1]:
+            raise ValueError("Grid transform must be a valid Affine matrix")
+        self.transform = Affine(*xform[:6])
+
+    def points(self, ring: bool = False) -> CoordList:
+        ny, nx = (float(dim) for dim in self.shape)
+        pts = [(0.0, 0.0), (nx, 0.0), (nx, ny), (0.0, ny)]
+        if ring:
+            pts += pts[:1]
+        return [self.transform * pt for pt in pts]
+
+    def ref_points(self) -> Dict[str, Dict[str, float]]:
+        nn = ['ul', 'ur', 'lr', 'll']
+        return {n: dict(x=x, y=y)
+                for n, (x, y) in zip(nn, self.points())}
+
+    def polygon(self, crs: Optional[SomeCRS] = None) -> Geometry:
+        return polygon(self.points(ring=True), crs=crs)
 
 
-def grid2points(grid: Dict[str, Any],
-                ring: bool = False) -> CoordList:
-    grid = _norm_grid(grid)
-
-    ny, nx = (float(dim) for dim in grid.shape)
-    transform = grid.transform
-    pts = [(0.0, 0.0), (nx, 0.0), (nx, ny), (0.0, ny)]
-    if ring:
-        pts += pts[:1]
-    return [transform*pt for pt in pts]
-
-
-def grid2ref_points(grid: Dict[str, Any]) -> Dict[str, Any]:
-    nn = ['ul', 'ur', 'lr', 'll']
-    return {n: dict(x=x, y=y)
-            for n, (x, y) in zip(nn, grid2points(grid))}
-
-
-def grid2polygon(grid: Dict[str, Any], crs: SomeCRS) -> Geometry:
-    return polygon(grid2points(grid, ring=True), crs)
-
-
-def eo3_lonlat_bbox(doc: Dict[str, Any],
+def eo3_lonlat_bbox(grids: Iterable[EO3Grid],
+                    crs: CRS,
+                    valid_data: Optional[Geometry] = None,
                     resolution: Optional[float] = None) -> BoundingBox:
-    """ Compute bounding box in Lon/Lat for a given EO3 document.
+    """ Compute bounding box for all grids in Lon/Lat
     """
-    crs = doc.get('crs')
-    grids = doc.get('grids')
+    if valid_data is not None:
+        return lonlat_bounds(valid_data, resolution=resolution)
 
-    if crs is None or grids is None:
-        raise ValueError("Input must have crs and grids")
-
-    crs = CRS(crs)
-    geom = doc.get('geometry', None)
-    if geom is not None:
-        geom = Geometry(geom, crs)
-        return lonlat_bounds(geom, resolution=resolution)
-
-    bounds = [lonlat_bounds(grid2polygon(grid, crs), resolution=resolution)
-              for grid in grids.values()]
-
-    return bbox_union(bounds)
+    all_grids_extent = reduce(lambda x, y: x.union(y), (grid.polygon(crs) for grid in grids))
+    return lonlat_bounds(all_grids_extent, resolution=resolution)
 
 
 def eo3_grid_spatial(doc: Dict[str, Any],
-                     resolution: Optional[float] = None) -> Dict[str, Any]:
+                     resolution: Optional[float] = None,
+                     grid_name: str = "default") -> Dict[str, Any]:
     """Using doc[grids|crs|geometry] compute EO3 style grid spatial:
 
     Note that `geo_ref_points` are set to the 4 corners of the default grid
     only, while lon/lat bounds are computed using all the grids, unless tighter
     valid region is defined via `geometry` key, in which case it is used to
     determine lon/lat bounds instead.
+    Uses the default grid.
 
     inputs:
+
     ```
     crs: "<:str>"
     geometry: <:GeoJSON object>  # optional
@@ -120,25 +118,32 @@ def eo3_grid_spatial(doc: Dict[str, Any],
     ```
 
     """
-    grid = toolz.get_in(['grids', 'default'], doc, None)
+    gridspecs = doc.get("grids", {})
     crs = doc.get('crs', None)
-    if crs is None or grid is None:
-        raise ValueError("Input must have crs and grids.default")
+    if crs is None or not gridspecs:
+        raise ValueError("Input must have crs and grids.")
+    grids = {name: EO3Grid(grid_spec) for name, grid_spec in gridspecs.items()}
+    grid = grids.get(grid_name)
+    if not grid:
+        raise ValueError(f"Input must have grids.{grid_name}")
 
     geometry = doc.get('geometry')
-
     if geometry is not None:
-        valid_data = dict(valid_data=geometry)
+        valid_data: Dict[str, Any] = dict(valid_data=geometry)
+        valid_geom: Optional[Geometry] = polygon(valid_data["valid_data"]["coordinates"][0], crs=crs)
     else:
-        valid_data = {}
+        valid_data = dict(valid_data=grid.polygon().json)
+        valid_geom = None
 
     oo = dict(grid_spatial=dict(projection={
         'spatial_reference': crs,
-        'geo_ref_points': grid2ref_points(grid),
+        'geo_ref_points': grid.ref_points(),
         **valid_data,
     }))
 
-    x1, y1, x2, y2 = eo3_lonlat_bbox(doc, resolution=resolution)
+    x1, y1, x2, y2 = eo3_lonlat_bbox(grids.values(), crs,
+                                     valid_data=valid_geom,
+                                     resolution=resolution)
     oo['extent'] = dict(lon=dict(begin=x1, end=x2),
                         lat=dict(begin=y1, end=y2))
     return oo
@@ -179,6 +184,25 @@ def is_doc_eo3(doc: Dict[str, Any]) -> bool:
     raise ValueError(f'Unsupported dataset schema: {schema!r}')
 
 
+def is_doc_geo(doc: Dict[str, Any], check_eo3: bool = True) -> bool:
+    """ Is this document geospatial?
+
+    :param doc: Parsed ODC Dataset metadata document
+    :param check_url: Set to false to skip the EO3 check and assume doc isn't EO3.
+
+    :returns:
+        True if this document specifies geospatial dimensions
+        False if this document does not specify geospatial dimensions (e.g. telemetry only)
+
+    :raises ValueError: For an unsupported document
+    """
+    # EO3 is geospatial
+    if check_eo3 and is_doc_eo3(doc):
+        return True
+    # Does this cover EO legacy datasets ok? at all??
+    return "extent" in doc or "grid_spatial" in doc
+
+
 def prep_eo3(doc: Dict[str, Any],
              auto_skip: bool = False,
              resolution: Optional[float] = None) -> Dict[str, Any]:
@@ -194,6 +218,11 @@ def prep_eo3(doc: Dict[str, Any],
         if not is_doc_eo3(doc):
             return doc
 
+    def stringify(u: Optional[Union[str, UUID]]) -> Optional[str]:
+        return u if isinstance(u, str) else str(u) if u else None
+
+    doc['id'] = stringify(doc.get('id', None))
+
     doc = add_eo3_parts(doc, resolution=resolution)
     lineage = doc.pop('lineage', {})
 
@@ -205,11 +234,11 @@ def prep_eo3(doc: Dict[str, Any],
         if isinstance(uuids, dict) or isinstance(uuids[0], dict):
             raise ValueError("Embedded lineage not supported for eo3 metadata types")
         if len(uuids) == 1:
-            return {name: {'id': uuids[0]}}
+            return {name: {'id': stringify(uuids[0])}}
 
         out = {}
         for idx, uuid in enumerate(uuids, start=1):
-            out[name+str(idx)] = {'id': uuid}
+            out[name+str(idx)] = {'id': stringify(uuid)}
         return out
 
     sources = {}
