@@ -8,7 +8,6 @@ Core classes used across modules.
 
 import logging
 import math
-import warnings
 from collections import OrderedDict
 from datetime import datetime
 from pathlib import Path
@@ -20,8 +19,9 @@ from typing import Optional, List, Mapping, Any, Dict, Tuple, Iterator, Iterable
 from urllib.parse import urlparse
 from datacube.utils import geometry, without_lineage_sources, parse_time, cached_property, uri_to_local_path, \
     schema_validated, DocReader
+from datacube.index.eo3 import is_doc_eo3
 from .fields import Field, get_dataset_fields
-from ._base import Range, ranges_overlap
+from ._base import Range, ranges_overlap  # noqa: F401
 
 _LOG = logging.getLogger(__name__)
 
@@ -43,14 +43,14 @@ class Dataset:
     """
 
     def __init__(self,
-                 type_: 'DatasetType',
+                 type_: 'Product',
                  metadata_doc: Dict[str, Any],
                  uris: Optional[List[str]] = None,
                  sources: Optional[Mapping[str, 'Dataset']] = None,
                  indexed_by: Optional[str] = None,
                  indexed_time: Optional[datetime] = None,
                  archived_time: Optional[datetime] = None):
-        assert isinstance(type_, DatasetType)
+        assert isinstance(type_, Product)
 
         self.type = type_
 
@@ -73,6 +73,10 @@ class Dataset:
         # When the dataset was archived. Null it not archived.
         self.archived_time = archived_time
 
+    @property
+    def is_eo3(self) -> bool:
+        return is_doc_eo3(self.metadata_doc)
+
     @property
     def metadata_type(self) -> 'MetadataType':
         return self.type.metadata_type
@@ -231,28 +235,6 @@ class Dataset:
         crs = projection.get('spatial_reference', None)
         if crs:
             return geometry.CRS(str(crs))
-
-        # Try to infer CRS
-        zone_ = projection.get('zone')
-        datum_ = projection.get('datum')
-        if zone_ and datum_:
-            warnings.warn("Using zone/datum to specify CRS is deprecated",
-                          category=DeprecationWarning)
-            try:
-                # TODO: really need CRS specified properly in agdc-metadata.yaml
-                if datum_ == 'GDA94':
-                    return geometry.CRS('EPSG:283' + str(abs(zone_)))
-                if datum_ == 'WGS84':
-                    if zone_[-1] == 'S':
-                        return geometry.CRS('EPSG:327' + str(abs(int(zone_[:-1]))))
-                    else:
-                        return geometry.CRS('EPSG:326' + str(abs(int(zone_[:-1]))))
-            except geometry.CRSError:
-                # We still return None, as they didn't specify a CRS explicitly...
-                _LOG.warning(
-                    "Can't figure out projection: possibly invalid zone (%r) for datum (%r).", zone_, datum_
-                )
-
         return None
 
     @cached_property
@@ -392,7 +374,7 @@ class MetadataType:
 
 
 @schema_validated(SCHEMA_PATH / 'dataset-type-schema.yaml')
-class DatasetType:
+class Product:
     """
     Product definition
 
@@ -696,7 +678,7 @@ class DatasetType:
         return row
 
     def __str__(self) -> str:
-        return "DatasetType(name={name!r}, id_={id!r})".format(id=self.id, name=self.name)
+        return "Product(name={name!r}, id_={id!r})".format(id=self.id, name=self.name)
 
     def __repr__(self) -> str:
         return self.__str__()
@@ -716,6 +698,10 @@ class DatasetType:
         return hash(self.name)
 
 
+# Type alias for backwards compatibility
+DatasetType = Product
+
+
 @schema_validated(SCHEMA_PATH / 'ingestor-config-type-schema.yaml')
 class IngestorConfig:
     """
