@@ -22,7 +22,7 @@ from datacube.index.eo3 import prep_eo3  # type: ignore[attr-defined]
 from datacube.index import Index
 from datacube.model import Dataset
 from datacube.ui import click as ui
-from datacube.ui.click import cli
+from datacube.ui.click import cli, print_help_msg
 from datacube.ui.common import ui_path_doc_stream
 from datacube.utils import changes, SimpleDocNav
 from datacube.utils.serialise import SafeDatacubeDumper
@@ -143,8 +143,6 @@ def load_datasets_for_update(doc_stream, index):
                     'specified with this option, '
                     'you can supply several by repeating this option with a new product name'),
               multiple=True)
-@click.option('--auto-match', '-a', help="Deprecated don't use it, it's a no-op",
-              is_flag=True, default=False, hidden=True)
 @click.option('--auto-add-lineage/--no-auto-add-lineage', is_flag=True, default=True,
               help=('Default behaviour is to automatically add lineage datasets if they are missing from the database, '
                     'but this can be disabled if lineage is expected to be present in the DB, '
@@ -164,13 +162,17 @@ def load_datasets_for_update(doc_stream, index):
 @ui.pass_index()
 def index_cmd(index, product_names,
               exclude_product_names,
-              auto_match,
               auto_add_lineage,
               verify_lineage,
               dry_run,
               ignore_lineage,
               confirm_ignore_lineage,
               dataset_paths):
+
+    if not dataset_paths:
+        print_help_msg(index_cmd)
+        sys.exit(1)
+
     if confirm_ignore_lineage is False and ignore_lineage is True:
         if sys.stdin.isatty():
             confirmed = click.confirm("Requested to skip lineage information, Are you sure?", default=False)
@@ -183,9 +185,6 @@ def index_cmd(index, product_names,
 
         confirm_ignore_lineage = True
 
-    if auto_match is True:
-        _LOG.warning("--auto-match option is deprecated, update your scripts, behaviour is the same without it")
-
     try:
         ds_resolve = Doc2Dataset(index,
                                  product_names,
@@ -241,7 +240,7 @@ def parse_update_rules(keys_that_can_change):
               default='keep',
               help=dedent('''
               What to do with previously recorded dataset location(s)
-              
+
               \b
               - 'keep': keep as alternative location [default]
               - 'archive': mark as archived
@@ -249,6 +248,10 @@ def parse_update_rules(keys_that_can_change):
 @click.argument('dataset-paths', nargs=-1)
 @ui.pass_index()
 def update_cmd(index, keys_that_can_change, dry_run, location_policy, dataset_paths):
+    if not dataset_paths:
+        print_help_msg(update_cmd)
+        sys.exit(1)
+
     def loc_action(action, new_ds, existing_ds, action_name):
         if len(existing_ds.uris) == 0:
             return None
@@ -412,6 +415,10 @@ def info_cmd(index: Index, show_sources: bool, show_derived: bool,
              f: str,
              max_depth: int,
              ids: Iterable[str]) -> None:
+    if not ids:
+        print_help_msg(info_cmd)
+        sys.exit(1)
+
     # Using an array wrapper to get around the lack of "nonlocal" in py2
     missing_datasets = [0]
 
@@ -478,6 +485,10 @@ def uri_search_cmd(index: Index, paths: List[str], search_mode):
 
     PATHS may be either file paths or URIs
     """
+    if not paths:
+        print_help_msg(uri_search_cmd)
+        sys.exit(1)
+
     if search_mode == 'guess':
         # This is what the API expects. I think it should be changed.
         search_mode = None
@@ -493,16 +504,22 @@ def uri_search_cmd(index: Index, paths: List[str], search_mode):
 @click.option('--archive-derived', '-d', help='Also recursively archive derived datasets', is_flag=True, default=False)
 @click.option('--dry-run', help="Don't archive. Display datasets that would get archived",
               is_flag=True, default=False)
-@click.option('--all', "all_ds", help="Ignore id list - archive ALL non-archived datasets  (warning: may be slow on large databases)",
+@click.option('--all', "all_ds",
+              help="Ignore id list - archive ALL non-archived datasets  (warning: may be slow on large databases)",
               is_flag=True, default=False)
 @click.argument('ids', nargs=-1)
 @ui.pass_index()
 def archive_cmd(index: Index, archive_derived: bool, dry_run: bool, all_ds: bool, ids: List[str]):
+    if not ids and not all_ds:
+        print_help_msg(archive_cmd)
+        sys.exit(1)
+
     derived_dataset_ids: List[UUID] = []
     if all_ds:
         datasets_for_archive = {dsid: True for dsid in index.datasets.get_all_dataset_ids(archived=False)}
     else:
-        datasets_for_archive = {UUID(dataset_id): exists for dataset_id, exists in zip(ids, index.datasets.bulk_has(ids))}
+        datasets_for_archive = {UUID(dataset_id): exists
+                                for dataset_id, exists in zip(ids, index.datasets.bulk_has(ids))}
 
         if False in datasets_for_archive.values():
             for dataset_id, exists in datasets_for_archive.items():
@@ -534,11 +551,17 @@ def archive_cmd(index: Index, archive_derived: bool, dry_run: bool, all_ds: bool
               help="Only restore derived datasets that were archived "
                    "this recently to the original dataset",
               default=10 * 60)
-@click.option('--all', "all_ds", help="Ignore id list - restore ALL archived datasets  (warning: may be slow on large databases)",
+@click.option('--all', "all_ds",
+              help="Ignore id list - restore ALL archived datasets  (warning: may be slow on large databases)",
               is_flag=True, default=False)
 @click.argument('ids', nargs=-1)
 @ui.pass_index()
-def restore_cmd(index: Index, restore_derived: bool, derived_tolerance_seconds: int, dry_run: bool, all_ds: bool, ids: List[str]):
+def restore_cmd(index: Index, restore_derived: bool, derived_tolerance_seconds: int,
+                dry_run: bool, all_ds: bool, ids: List[str]):
+    if not ids and not all_ds:
+        print_help_msg(restore_cmd)
+        sys.exit(1)
+
     tolerance = datetime.timedelta(seconds=derived_tolerance_seconds)
     if all_ds:
         ids = index.datasets.get_all_dataset_ids(archived=True)  # type: ignore[assignment]
@@ -576,15 +599,21 @@ def restore_cmd(index: Index, restore_derived: bool, derived_tolerance_seconds:
 @dataset_cmd.command('purge', help="Purge archived datasets")
 @click.option('--dry-run', help="Don't archive. Display datasets that would get archived",
               is_flag=True, default=False)
-@click.option('--all', "all_ds", help="Ignore id list - purge ALL archived datasets  (warning: may be slow on large databases)",
+@click.option('--all', "all_ds",
+              help="Ignore id list - purge ALL archived datasets  (warning: may be slow on large databases)",
               is_flag=True, default=False)
 @click.argument('ids', nargs=-1)
 @ui.pass_index()
 def purge_cmd(index: Index, dry_run: bool, all_ds: bool, ids: List[str]):
+    if not ids and not all_ds:
+        print_help_msg(purge_cmd)
+        sys.exit(1)
+
     if all_ds:
         datasets_for_archive = {dsid: True for dsid in index.datasets.get_all_dataset_ids(archived=True)}
     else:
-        datasets_for_archive = {UUID(dataset_id): exists for dataset_id, exists in zip(ids, index.datasets.bulk_has(ids))}
+        datasets_for_archive = {UUID(dataset_id): exists
+                                for dataset_id, exists in zip(ids, index.datasets.bulk_has(ids))}
 
         # Check for non-existent datasets
         if False in datasets_for_archive.values():
