@@ -5,7 +5,6 @@
 import datetime
 
 import pytest
-from uuid import UUID
 from datacube.testutils import gen_dataset_test_dag
 
 from datacube.utils import InvalidDocException, read_documents, SimpleDocNav
@@ -549,7 +548,23 @@ def test_memory_dataset_add(dataset_add_configs, mem_index_fresh):
     for id_ in ds_ids:
         assert id_ in loc_ids
     ds_ = SimpleDocNav(gen_dataset_test_dag(1, force_tree=True))
-    assert UUID(ds_.id) in ds_ids
+    assert ds_.id in ds_ids
     ds_from_idx = idx.datasets.get(ds_.id, include_sources=True)
-    assert str(ds_from_idx.sources['ab'].id) == ds_.sources['ab'].id
-    assert str(ds_from_idx.sources['ac'].sources["cd"].id) == ds_.sources['ac'].sources['cd'].id
+    assert ds_from_idx.sources['ab'].id == ds_.sources['ab'].id
+    assert ds_from_idx.sources['ac'].sources["cd"].id == ds_.sources['ac'].sources['cd'].id
+
+
+def test_mem_transactions(mem_index_fresh):
+    trans = mem_index_fresh.index.transaction()
+    assert not trans.active
+    trans.begin()
+    assert trans.active
+    trans.commit()
+    assert not trans.active
+    trans.begin()
+    assert mem_index_fresh.index.thread_transaction() == trans
+    with pytest.raises(ValueError):
+        trans.begin()
+    trans.rollback()
+    assert not trans.active
+    assert mem_index_fresh.index.thread_transaction() is None
