@@ -75,6 +75,74 @@ def test_load_data(tmpdir):
     assert progress_call_data == [(1, 2), (2, 2)]
 
 
+def test_load_data_with_url_mangling(tmpdir):
+    actual_tmpdir = Path(str(tmpdir))
+    recorded_tmpdir = Path(str(tmpdir / "not" / "actual" / "location"))
+
+    def url_mangler(raw):
+        actual_uri_root = actual_tmpdir.absolute().as_uri()
+        recorded_uri_root = recorded_tmpdir.absolute().as_uri()
+        return raw.replace(recorded_uri_root, actual_uri_root)
+
+    group_by = query_group_by('time')
+    spatial = dict(resolution=(15, -15),
+                   offset=(11230, 1381110),)
+
+    nodata = -999
+    aa = mk_test_image(96, 64, 'int16', nodata=nodata)
+
+    ds, gbox = gen_tiff_dataset([SimpleNamespace(name='aa', values=aa, nodata=nodata)],
+                                tmpdir,
+                                prefix='ds1-',
+                                timestamp='2018-07-19',
+                                base_folder_of_record=recorded_tmpdir,
+                                **spatial)
+    assert ds.time is not None
+
+    ds2, _ = gen_tiff_dataset([SimpleNamespace(name='aa', values=aa, nodata=nodata)],
+                              tmpdir,
+                              prefix='ds2-',
+                              timestamp='2018-07-19',
+                              base_folder_of_record=recorded_tmpdir,
+                              **spatial)
+    assert ds.time is not None
+    assert ds.time == ds2.time
+
+    sources = Datacube.group_datasets([ds], 'time')
+    sources2 = Datacube.group_datasets([ds, ds2], group_by)
+
+    mm = ['aa']
+    mm = [ds.type.measurements[k] for k in mm]
+
+    ds_data = Datacube.load_data(sources, gbox, mm, patch_url=url_mangler)
+    assert ds_data.aa.nodata == nodata
+    np.testing.assert_array_equal(aa, ds_data.aa.values[0])
+
+    ds_data = Datacube.load_data(sources, gbox, mm, dask_chunks={'x': 8, 'y': 8}, patch_url=url_mangler)
+    assert ds_data.aa.nodata == nodata
+    np.testing.assert_array_equal(aa, ds_data.aa.values[0])
+
+    custom_fuser_call_count = 0
+
+    def custom_fuser(dest, delta):
+        nonlocal custom_fuser_call_count
+        custom_fuser_call_count += 1
+        dest[:] += delta
+
+    progress_call_data = []
+
+    def progress_cbk(n, nt):
+        progress_call_data.append((n, nt))
+
+    ds_data = Datacube.load_data(sources2, gbox, mm, fuse_func=custom_fuser,
+                                 progress_cbk=progress_cbk, patch_url=url_mangler)
+    assert ds_data.aa.nodata == nodata
+    assert custom_fuser_call_count > 0
+    np.testing.assert_array_equal(nodata + aa + aa, ds_data.aa.values[0])
+
+    assert progress_call_data == [(1, 2), (2, 2)]
+
+
 def test_load_data_cbk(tmpdir):
     from datacube.api import TerminateCurrentLoad
 
